@@ -35,6 +35,11 @@
 #include <iostream>   // for cout, endl
 #include <iomanip>
 
+#include <stack>
+
+namespace Glom
+{
+
 template<class T_Element>
 class predicate_UsesRelationshipHasRelationships
 {
@@ -45,10 +50,13 @@ public:
   {
   }
 
-  predicate_UsesRelationshipHasRelationships(const sharedptr<const UsesRelationship> uses_relationship_name)
+  predicate_UsesRelationshipHasRelationships(const sharedptr<const UsesRelationship> uses_relationship_name, bool first_level_only = false)
   : m_relationship_name(uses_relationship_name->get_relationship_name()),
     m_related_relationship_name(uses_relationship_name->get_related_relationship_name())
   {
+    //If first_level_only, search for relationships that have the same top-level relationship, but have no related relationship.
+    if(first_level_only)
+      m_related_relationship_name = Glib::ustring();
   }
 
   virtual ~predicate_UsesRelationshipHasRelationships()
@@ -75,7 +83,7 @@ protected:
 };
 
 
-Glib::ustring GlomUtils::trim_whitespace(const Glib::ustring& text)
+Glib::ustring Utils::trim_whitespace(const Glib::ustring& text)
 {
   //TODO_Performance:
 
@@ -119,7 +127,7 @@ Glib::ustring GlomUtils::trim_whitespace(const Glib::ustring& text)
   return result;
 }
 
-Glib::ustring GlomUtils::string_replace(const Glib::ustring& src, const Glib::ustring search_for, const Glib::ustring& replace_with)
+Glib::ustring Utils::string_replace(const Glib::ustring& src, const Glib::ustring search_for, const Glib::ustring& replace_with)
 {
   std::string result = src;
 
@@ -168,8 +176,49 @@ Glib::ustring GlomUtils::string_replace(const Glib::ustring& src, const Glib::us
 }
 
 
+Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const type_sort_clause& sort_clause)
+{
+  //TODO_Performance:
+  type_vecConstLayoutFields constFieldsToGet;
+  for(type_vecLayoutFields::const_iterator iter = fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
+  {
+    constFieldsToGet.push_back(*iter);
+  }
 
-Glib::ustring GlomUtils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const type_sort_clause& sort_clause)
+  return build_sql_select_with_where_clause(table_name, constFieldsToGet, where_clause, sort_clause);
+}
+
+
+static typedef std::list< sharedptr<const UsesRelationship> > type_list_relationships;
+
+static void add_to_relationships_list(type_list_relationships& list_relationships, const sharedptr<const LayoutItem_Field>& layout_item)
+{
+  if(!(layout_item->get_has_relationship_name()))
+    return;
+
+  //If this is a related relationship, add the first-level relationship too, so that the related relationship can be defined in terms of it:
+  type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item, true /* top_level_only */) );
+  if(iterFind == list_relationships.end()) //If the table is not yet in the list:
+  {
+    sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
+    uses_rel->set_relationship(layout_item->get_relationship());
+    list_relationships.push_front(uses_rel); //These need to be at the front, so that related relationships can use them later in the SQL statement.
+  }
+
+  //Add the relationship to the list:
+  iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item) );
+  if(iterFind == list_relationships.end()) //If the table is not yet in the list:
+  {
+    sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
+    uses_rel->set_relationship(layout_item->get_relationship());
+    uses_rel->set_related_relationship(layout_item->get_related_relationship());
+    list_relationships.push_back(uses_rel);
+  }
+
+ 
+}
+
+Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const type_sort_clause& sort_clause)
 {
   Glib::ustring result;
 
@@ -177,55 +226,30 @@ Glib::ustring GlomUtils::build_sql_select_with_where_clause(const Glib::ustring&
   typedef std::list< sharedptr<const UsesRelationship> > type_list_relationships;
   type_list_relationships list_relationships;
 
-  for(type_vecLayoutFields::const_iterator iter = fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
+  for(type_vecConstLayoutFields::const_iterator iter = fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
   {
-    sharedptr<LayoutItem_Field> layout_item = *iter;
-
-    if(layout_item->get_has_relationship_name())
-    {
-      //Add the relationship to the list:
-      type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item) );
-      if(iterFind == list_relationships.end()) //If the table is not yet in the list:
-      {
-        sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
-        uses_rel->set_relationship(layout_item->get_relationship());
-        uses_rel->set_related_relationship(layout_item->get_related_relationship());
-        list_relationships.push_back(uses_rel);
-      }
-    }
+    sharedptr<const LayoutItem_Field> layout_item = *iter;
+    add_to_relationships_list(list_relationships, layout_item);
   }
 
   for(type_sort_clause::const_iterator iter = sort_clause.begin(); iter != sort_clause.end(); ++iter)
   {
     sharedptr<const LayoutItem_Field> layout_item = iter->first;
-
-    if(layout_item->get_has_relationship_name())
-    {
-      //Add the relationship to the list:
-      type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item) );
-      if(iterFind == list_relationships.end()) //If the table is not yet in the list:
-      {
-        sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
-        uses_rel->set_relationship(layout_item->get_relationship());
-        uses_rel->set_related_relationship(layout_item->get_related_relationship());
-        list_relationships.push_back(uses_rel);
-      }
-    }
+    add_to_relationships_list(list_relationships, layout_item);
   }
 
 
-
   Glib::ustring sql_part_fields;
   Glib::ustring sql_part_from;
 
-  for(type_vecLayoutFields::const_iterator iter = fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
+  for(type_vecConstLayoutFields::const_iterator iter = fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
   {
     Glib::ustring one_sql_part;
 
-    sharedptr<LayoutItem_Field> layout_item = *iter;
+    sharedptr<const LayoutItem_Field> layout_item = *iter;
 
     bool is_summary = false;
-    LayoutItem_FieldSummary* fieldsummary = dynamic_cast<LayoutItem_FieldSummary*>(layout_item.obj());
+    const LayoutItem_FieldSummary* fieldsummary = dynamic_cast<const LayoutItem_FieldSummary*>(layout_item.obj());
     if(fieldsummary)
       is_summary = true;
 
@@ -258,7 +282,7 @@ Glib::ustring GlomUtils::build_sql_select_with_where_clause(const Glib::ustring&
 
   if(sql_part_fields.empty())
   {
-    std::cerr << "GlomUtils::build_sql_select_with_where_clause(): sql_part_fields.empty(): fieldsToGet.size()=" << fieldsToGet.size() << std::endl;
+    std::cerr << "Utils::build_sql_select_with_where_clause(): sql_part_fields.empty(): fieldsToGet.size()=" << fieldsToGet.size() << std::endl;
     return result;
   }
   else
@@ -321,18 +345,33 @@ Glib::ustring GlomUtils::build_sql_select_with_where_clause(const Glib::ustring&
   return result;
 }
 
-Glib::ustring GlomUtils::build_sql_select_with_key(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value)
+
+Glib::ustring Utils::build_sql_select_with_key(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value)
 {
-  if(!GlomConversions::value_is_empty(key_value)) //If there is a record to show:
+  //TODO_Performance:
+  type_vecConstLayoutFields constFieldsToGet;
+  for(type_vecLayoutFields::const_iterator iter = fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
+  {
+    constFieldsToGet.push_back(*iter);
+  }
+
+  return build_sql_select_with_key(table_name, constFieldsToGet, key_field, key_value);
+
+
+}
+
+Glib::ustring Utils::build_sql_select_with_key(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value)
+{
+  if(!Conversions::value_is_empty(key_value)) //If there is a record to show:
   {
     const Glib::ustring where_clause = "\"" + table_name + "\".\"" + key_field->get_name() + "\" = " + key_field->sql(key_value);
-    return GlomUtils::build_sql_select_with_where_clause(table_name, fieldsToGet, where_clause);
+    return Utils::build_sql_select_with_where_clause(table_name, fieldsToGet, where_clause);
   }
 
   return Glib::ustring();
 }
 
-GlomUtils::type_list_values_with_second GlomUtils::get_choice_values(const sharedptr<const LayoutItem_Field>& field)
+Utils::type_list_values_with_second Utils::get_choice_values(const sharedptr<const LayoutItem_Field>& field)
 {
   type_list_values_with_second list_values;
 
@@ -390,13 +429,13 @@ GlomUtils::type_list_values_with_second GlomUtils::get_choice_values(const share
 }
 
 
-Glib::ustring GlomUtils::create_name_from_title(const Glib::ustring& title)
+Glib::ustring Utils::create_name_from_title(const Glib::ustring& title)
 {
   Glib::ustring result = string_replace(title, " ", "");
   return result.lowercase(); //TODO: Maybe they need to be ASCII (not UTF8)?
 }
 
-Glib::ustring GlomUtils::string_escape_underscores(const Glib::ustring& text)
+Glib::ustring Utils::string_escape_underscores(const Glib::ustring& text)
 {
   Glib::ustring result;
   for(Glib::ustring::const_iterator iter = text.begin(); iter != text.end(); ++iter)
@@ -413,7 +452,7 @@ Glib::ustring GlomUtils::string_escape_underscores(const Glib::ustring& text)
 /** Get just the first part of a locale, such as de_DE, 
  * ignoring, for instance, .UTF-8 or @euro at the end.
  */
-Glib::ustring GlomUtils::locale_simplify(const Glib::ustring& locale_id)
+Glib::ustring Utils::locale_simplify(const Glib::ustring& locale_id)
 {
   Glib::ustring result = locale_id;
 
@@ -434,7 +473,7 @@ Glib::ustring GlomUtils::locale_simplify(const Glib::ustring& locale_id)
   return result;
 }
 
-Glib::ustring GlomUtils::locale_language_id(const Glib::ustring& locale_id)
+Glib::ustring Utils::locale_language_id(const Glib::ustring& locale_id)
 {
   Glib::ustring result;
 
@@ -447,7 +486,7 @@ Glib::ustring GlomUtils::locale_language_id(const Glib::ustring& locale_id)
   return result;
 }
 
-Glib::ustring GlomUtils::create_local_image_uri(const Gnome::Gda::Value& value)
+Glib::ustring Utils::create_local_image_uri(const Gnome::Gda::Value& value)
 {
   static guint m_temp_image_uri_number = 0;
 
@@ -483,13 +522,216 @@ Glib::ustring GlomUtils::create_local_image_uri(const Gnome::Gda::Value& value)
       }
     }
     else
-       std::cerr << "GlomUtils::create_local_image_uri(): binary GdaValue contains no data." << std::endl;
+       std::cerr << "Utils::create_local_image_uri(): binary GdaValue contains no data." << std::endl;
   }
   //else
-  //  std::cerr << "GlomUtils::create_local_image_uri(): type != BINARY" << std::endl;
+  //  std::cerr << "Utils::create_local_image_uri(): type != BINARY" << std::endl;
 
   if(result.empty())
     result = "/tmp/glom_report_image_invalid.png";
 
   return ("file://" + result);
 }
+
+Glib::ustring Utils::string_from_decimal(guint decimal)
+{
+  //TODO_Performance:
+
+  std::stringstream stream;
+  stream << decimal;
+
+  Glib::ustring result;
+  stream >> result;
+
+  return result;
+}
+
+guint Utils::decimal_from_string(const Glib::ustring& str)
+{
+  //TODO_Performance:
+
+  //Convert it to a numeric type:
+  std::stringstream stream;
+  stream << str;
+  guint id_numeric = 0;
+  stream >> id_numeric;
+
+  return id_numeric;
+}
+
+Glib::ustring Utils::title_from_string(const Glib::ustring& text)
+{
+  Glib::ustring result;
+
+  bool capitalise_next_char = true;
+  for(Glib::ustring::const_iterator iter = text.begin(); iter != text.end(); ++iter)
+  {
+    const gunichar& ch = *iter;
+    if(ch == '_') //Replace _ with space.
+    {
+      capitalise_next_char = true; //Capitalise all words.
+      result += " ";
+    }
+    else
+    {
+      if(capitalise_next_char)
+        result += Glib::Unicode::toupper(*iter);
+      else
+        result += *iter;
+
+      capitalise_next_char = false;
+    }
+  }
+
+  return result;
+}
+
+Utils::type_vecStrings Utils::string_separate(const Glib::ustring& str, const Glib::ustring& separator, bool ignore_quoted_separator)
+{
+  type_vecStrings result;
+
+  const Glib::ustring::size_type size = str.size();
+  const Glib::ustring::size_type size_separator = separator.size();
+
+  //A stack of quotes, so that we can handle nested quotes, whether they are " or ':
+  typedef std::stack<Glib::ustring> type_queue_quotes; 
+  type_queue_quotes m_current_quotes;
+
+  Glib::ustring::size_type unprocessed_start = 0;
+  Glib::ustring::size_type item_start = 0;
+  while(unprocessed_start < size)
+  { 
+    //std::cout << "while unprocessed: un_processed_start=" << unprocessed_start << std::endl;
+    Glib::ustring::size_type posComma = str.find(separator, unprocessed_start);
+
+    Glib::ustring item;
+    if(posComma != Glib::ustring::npos)
+    {
+      //Check that the separator was not in quotes:
+      bool in_quotes = false;
+
+      if(ignore_quoted_separator)
+      {
+        //std::cout << "  debug: attempting to ignore quoted separators: " << separator << std::endl;
+       
+        Glib::ustring::size_type posLastQuote = unprocessed_start;
+
+        //std::cout << "    debug: posLastQuote=" << posLastQuote << std::endl;
+        //std::cout << "    debug: posComma=" << posComma << std::endl;
+ 
+  
+        bool bContinue = true;
+        while(bContinue && (posLastQuote < posComma))
+        {
+          //std::cout << "  continue" << std::endl;
+          Glib::ustring closing_quote; 
+          if(!m_current_quotes.empty())
+            closing_quote = m_current_quotes.top();
+
+          //std::cout << "   posLastQuote=" << posLastQuote << std::endl;
+          const Glib::ustring::size_type posSingleQuote = str.find("'", posLastQuote);
+          const Glib::ustring::size_type posDoubleQuote = str.find("\"", posLastQuote);
+
+         // std::cout << "   posSingleQuote=" << posSingleQuote << "posDoubleQuote=" << posDoubleQuote << std::endl;
+
+          //Which quote, if any, is first:
+          Glib::ustring::size_type posFirstQuote = posSingleQuote;
+          if( (posDoubleQuote != Glib::ustring::npos) && (posDoubleQuote < posFirstQuote) )
+	    posFirstQuote = posDoubleQuote;
+
+          //Ignore quotes that are _after_ the separator:
+          if( posFirstQuote >= posComma)
+            posFirstQuote = Glib::ustring::npos;
+
+          //std::cout << "   posFirstQuote=" << posFirstQuote << std::endl;
+
+          //If any quote character was found:
+          if(posFirstQuote != Glib::ustring::npos)
+          {
+            //std::cout << "quote found: posFirstQuote=" << posFirstQuote << std::endl;
+
+            //Which quote was it?
+            const Glib::ustring first_quote =  (posFirstQuote == posSingleQuote ? "'" : "\"");
+            //std::cout << "   first_quote=" << first_quote << std::endl;
+
+            //Was it an expected closing quote, if we expected any:
+            if(first_quote == closing_quote)
+            {
+              //std::cout << "   popping quote" << std::endl;
+              //Yes, so remove that quote from our stack, because we found the closing quote:
+              m_current_quotes.pop();
+            }
+            else
+            {
+              //std::cout << "   pushing quote" << std::endl;
+              //This must be an opening quote, so remember it:
+              m_current_quotes.push(first_quote);
+            }
+
+            posLastQuote = posFirstQuote + 1; //Do the next find after the quote.
+          }
+          else
+          {
+            //There were no quotes, or no closing quotes:
+            bContinue = false;
+          }
+        } //while(bContinue)
+
+        //If there were any unclosed quotes then this separator must have been in quotes:
+        if(!m_current_quotes.empty())
+          in_quotes = true;
+      } //If ignore_quoted_separator
+
+      if(!in_quotes) //or if we don't care about quotes.
+      {
+        //std::cout << "!in_quotes" << std::endl;
+
+        //Store this item, and start the next item after it:
+        item = str.substr(item_start, posComma - item_start);
+        //std::cout << "ITEM. pos_comma=" << posComma << ", ITEM= " << item << std::endl;
+	item_start = posComma + size_separator;
+      }
+      //else
+      //  std::cout << "in quotes." << std::endl;
+     
+      unprocessed_start = posComma + size_separator; //The while loops stops when this is empty.
+    }
+    else //if no separator found:
+    {
+        item = str.substr(item_start);
+        unprocessed_start = size; //Stop.
+    }
+
+    item = string_trim(item, " ");
+    result.push_back(item);
+  } //while
+
+  return result;
+}
+
+Glib::ustring Utils::string_trim(const Glib::ustring& str, const Glib::ustring& to_remove)
+{
+   Glib::ustring result = str;
+
+   //Remove from the start:
+   Glib::ustring::size_type posOpenBracket = result.find(to_remove);
+   if(posOpenBracket == 0)
+   {
+//g_warning("string_trim: before start trim: %s", result.c_str());
+      result = result.substr(to_remove.size());
+//g_warning("string_trim: after start trim: %s", result.c_str());
+   }
+
+   //Remove from the end:
+   Glib::ustring::size_type posCloseBracket = result.rfind(to_remove);
+   if(posCloseBracket == (result.size() - to_remove.size()))
+   {
+     //g_warning("string_trim: before end trim: %s", result.c_str());
+    result = result.substr(0, posCloseBracket);
+     //g_warning("string_trim: after end trim: %s", result.c_str());
+   }
+
+  return result;
+}
+
+} //namespace Glom
