@@ -26,6 +26,9 @@
 #include <glom/libglom/document/document_glom.h>
 #include "../box_db.h"
 
+namespace Glom
+{
+
 class Dialog_Layout :
   public Gtk::Dialog,
   public Base_DB //Give it access to the document, and to the database utilities
@@ -69,4 +72,6 @@ protected:
   bool m_modified;
 };
 
+} //namespace Glom
+
 #endif //GLOM_MODE_DATA_DIALOG_LAYOUT_H
