@@ -22,6 +22,9 @@
 #include <glibmm/i18n.h>
 #include "../application.h"
 
+namespace Glom
+{
+
 LayoutWidgetBase::LayoutWidgetBase()
 : m_pLayoutItem(0),
   m_pMenuPopup(0)
@@ -185,3 +188,9 @@ LayoutWidgetBase::type_signal_user_requested_layout_properties LayoutWidgetBase:
 {
   return m_signal_user_requested_layout_properties;
 }
+
+void LayoutWidgetBase::set_read_only(bool /* read_only */)
+{
+}
+
+} //namespace Glom
